<?php

namespace Drupal\monarch_search_block\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure Monarch Search Block settings for this site.
 */
class SearchBlockSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'monarch_search_block_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['monarch_search_block.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['placeholder'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Placeholder'),
      '#default_value' => $this->config('monarch_search_block.settings')->get('placeholder'),
      '#description' => $this->t('The placeholder text to display in the search box.'),
    ];

    $form['new_tab'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Open search results in a new tab'),
      '#default_value' => $this->config('monarch_search_block.settings')->get('new_tab'),
    ];

    $wrapper_id = 'search-block-services-wrapper';

    $form['services'] = [
      '#prefix' => '<div id="' . $wrapper_id . '">',
      '#suffix' => '</div>',
      '#type' => 'table',
      '#attributes' => [
        'id' => 'monarch-search-block-services',
      ],
      '#header' => [
        'title' => $this->t('Option Label'),
        'base_url' => $this->t('Base URL'),
        'param' => $this->t('Query Parameter'),
        'weight' => $this->t('Weight'),
        'remove' => $this->t('Remove'),
      ],
      '#empty' => $this->t('No search options configured.'),
      '#tabledrag' => [[
        'action' => 'order',
        'relationship' => 'sibling',
        'group' => 'table-sort-weight',
      ]],
    ];

    if ($form_state->isRebuilding()) {
      $services = $form_state->getValue('services');
    }
    else {
      $services = $this->config('monarch_search_block.settings')->get('services');
    }

    $num_services = $form_state->get('num_services');
    if (is_null($num_services)) {
      $num_services = count($services);
      $form_state->set('num_services', $num_services);
    }

    // Order by weight
    usort($services, function ($a, $b) {
      return $a['weight'] <=> $b['weight'];
    });

    for ($i = 0; $i < $num_services; $i++) {
      $form['services'][$i]['#attributes']['class'][] = 'draggable';

      $form['services'][$i]['title'] = [
        '#type' => 'textfield',
        '#required' => TRUE,
        '#default_value' => $services[$i]['title'] ?? '',
      ];

      $form['services'][$i]['base_url'] = [
        '#type' => 'textfield',
        '#required' => TRUE,
        '#default_value' => $services[$i]['base_url'] ?? '',
        '#description' => $this->t('The base URL for the search option. The query parameter will be appended to this URL.'),
      ];

      $form['services'][$i]['param'] = [
        '#type' => 'textfield',
        '#required' => TRUE,
        '#default_value' => $services[$i]['param'] ?? '',
        '#size' => 5,
        '#description' => $this->t('The query parameter to use for the search option. For example, \'q\' for Google, \'query\' for Wikipedia, etc.'),
      ];

      $form['services'][$i]['weight'] = [
        '#type' => 'weight',
        '#title' => $this->t('Weight for @title', ['@title' => $services[$i]['title']]),
        '#title_display' => 'invisible',
        '#default_value' => $services[$i]['weight'] ?? '',
        '#attributes' => [
          'class' => [
            'table-sort-weight',
          ],
        ],
      ];

      $form['services'][$i]['remove'] = [
        '#type' => 'submit',
        '#name' => 'remove_' . $i,
        '#value' => $this->t('Remove'),
        '#submit' => ['::removeCallback'],
        '#delta' => $i,
        '#ajax' => [
          'callback' => '::ajaxCallback',
          'wrapper' => $wrapper_id,
        ],
      ];
    }

    $form['add'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add search option'),
      '#submit' => ['::addOne'],
      '#ajax' => [
        'callback' => '::ajaxCallback',
        'wrapper' => $wrapper_id,
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * Add another set of search service fields.
   */
  public function addOne(array &$form, FormStateInterface $form_state) {
    $num_services = $form_state->get('num_services');
    $form_state->set('num_services', $num_services + 1);
    $form_state->setRebuild();
  }

  /**
   * Ajax callback handler for adding more search services.
   */
  public function ajaxCallback(array &$form, FormStateInterface $form_state) {
    return $form['services'];
  }

  /**
   * Remove a set of search service fields.
   */
  public function removeCallback(array &$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    $delta = $triggering_element['#delta'];
    $services = $form_state->getValue('services');
    unset($services[$delta]);
    $form_state->setValue('services', $services);

    $num_services = $form_state->get('num_services');
    $form_state->set('num_services', $num_services - 1);

    $form_state->setRebuild();
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $services = $form_state->getValue('services');
    $services = array_filter($services, function ($service) {
      return !empty($service['title']) && !empty($service['base_url']) && !empty($service['param']);
    });
    $form_state->setValue('services', $services);
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('monarch_search_block.settings')
      ->set('placeholder', $form_state->getValue('placeholder'))
      ->save();

    $this->config('monarch_search_block.settings')
      ->set('new_tab', $form_state->getValue('new_tab'))
      ->save();

    $services = $form_state->getValue('services');
    $this->config('monarch_search_block.settings')
      ->set('services', $services)
      ->save();

    parent::submitForm($form, $form_state);
  }

}
